//=========================================================================
//
//	FILE:				DragDrop_ContentsView.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_ScrollableView to display the contents of a CP_DragDrop operation.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 4 $
//						Latest Checkout:  $Date: 2005-09-09 22:34:33 -0700 (Fri, 09 Sep 2005) $
//
//=========================================================================

	// HelloWorld
#include "DragDrop_ContentsView.h"

	// CPLAT_Framework
#include "CP_Alerts.h"
#include "CP_Enums.h"
#include "CP_Graphics.h"
#include "CP_Utility.h"
#include "CP_Window.h"
#include "CP_WindowEvents.h"

const float kLeftMargin = 5;
const float kLeftMargin_Item = 50;
const float kLeftMargin_Flavor = 100;
const float kLineHeight = 16;
const float k2ndColumnOffset = 75;

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	DragDropContentsView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor. Create a pane with the upper left corner at inLocation, and
*	the content size of inSize.
*
*	@param		inSuperview			The subview handler that contains this view.
*	@param		inUpperLeft			Upper left corner of pane in releation to its superview.
*	@param		inContentSize		Content size of the view.
*	
*	@exception	none
*/
DragDropContentsView::DragDropContentsView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize)
		: CP_ScrollableView( inSuperview, inUpperLeft, inContentSize, true,
			CPLAT::CP_E_ViewFlags_SupportsDragAndDrop )
{
#if TARGET_OS_WIN32
	SetTargetWindow( GetContainingWindow()->GetOSWindow() );
	
	::DragAcceptFiles( GetOSControl(), true );
	
	InitEventHandler( GetContainingWindow()->GetEventHandler() );
#endif
}

//=========================================================================
//	FUNCTION:	~DragDropContentsView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
DragDropContentsView::~DragDropContentsView() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drag and Drop ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CanAcceptDrag
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Returns a bool if the view can accept the object being dragged over this
*	view. In this demo, since we want to look at the flavors of all items, we
*	just return true to indicate we can accept any item dragged over the window.
*
*	@param		inItem					Not used.
*	@param		inDragDrop				Not used.
*	
*	@exception	none
*
*	@return		bool					We always return true for this demo to
*										indicate any item can be dragging onto
*										the window.
*/
bool DragDropContentsView::OnCanAcceptDrag( UInt32 /*inItem*/, CPLAT::CP_DragDrop& /*inDragDrop*/ )
{
		// we accept all items here, since we are just displaying
		// the type of data contained.
	return( true );
}

//=========================================================================
//	FUNCTION:	HandleDragReceive
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when the user has dropped some item onto this view. In this demo
*	we will display all of the flavors in the item.
*
*	@param		inDragDrop				The CP_DragDrop object performing the
*										drop.
*	
*	@exception	none
*
*	@return		void
*/
void DragDropContentsView::HandleDragReceive( CPLAT::CP_DragDrop& inDragDrop )
{
	UInt16			itemCount = inDragDrop.GetCount();
	UInt16			x = 1;
	
		// remove previous entries
	if( fDroppedItems.size() > 0 )
		fDroppedItems.erase( fDroppedItems.begin(), fDroppedItems.end() );
	
		// store a copy of each item in the drop
	for( x = 1; x <= itemCount; x++ )
	{
		CPLAT::CP_DataStream		nextItem = inDragDrop.GetItem( x );
		
		fDroppedItems.push_back( nextItem );

		Refresh();
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Loops through all of the items in the dropped item, and displays the type
*	of flavor, and some information about each item. 
*	
*	@param		inCanvas			The destination CP_Canvas object.
*	@param		inUpdateArea		The area needing to be updated.
*	
*	@exception	none
*
*	@return		void
*/
void DragDropContentsView::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	CPLAT::CP_Rect			localBounds = GetLocalBounds();
	UInt32					numberDroppedItems = fDroppedItems.size();
	UInt32					itemCounter = 0;
	CPLAT::CP_Graphics		graphics( inCanvas );
	CPLAT::CP_Point			penLocation( kLeftMargin_Flavor, 20 );
	CPLAT::CP_String		outputString;
	CPLAT::CP_FontSpec		fontSpec;
		
	if( numberDroppedItems > 0 )
	{
			// output information about the drag drop
		outputString = "Number of items: ";
		penLocation.Set( kLeftMargin, penLocation.v() );
		graphics.DrawString( fontSpec, outputString, penLocation );
		penLocation.Set( kLeftMargin + 150, penLocation.v() );
		graphics.DrawString( fontSpec, CPLAT::CP_Utility::CP_UInt32ToString( numberDroppedItems ), penLocation );

			// advance to next line
		penLocation.Set( kLeftMargin, penLocation.v() + (kLineHeight * 2) );

		for( itemCounter = 0; itemCounter < numberDroppedItems; itemCounter++ )
		{
			CPLAT::CP_DataStream		dataStream = fDroppedItems[ itemCounter ];
			UInt32						dataStreamItemCounter = 1;
			UInt32						numberDataStreamItems = dataStream.GetCount();
			UInt32						flavorType = 0;
			char						buffer[ 256 ];
			
				// information about this drag item
			outputString = "Item #: " + CPLAT::CP_Utility::CP_UInt32ToString( itemCounter + 1 );
			penLocation.Set( kLeftMargin_Item, penLocation.v() );
			graphics.DrawString( fontSpec, outputString, penLocation );

				// advance to next line
			penLocation.Set( kLeftMargin, penLocation.v() + kLineHeight );

			outputString = "Number of flavors: ";
			penLocation.Set( kLeftMargin_Flavor, penLocation.v() );
			graphics.DrawString( fontSpec, outputString, penLocation );
			penLocation.Set( kLeftMargin_Flavor + 150, penLocation.v() );
			graphics.DrawString( fontSpec, CPLAT::CP_Utility::CP_UInt32ToString( numberDataStreamItems ), penLocation );

				// advance to next line
			penLocation.Set( kLeftMargin_Flavor, penLocation.v() + kLineHeight );

			for( dataStreamItemCounter = 1; dataStreamItemCounter <= numberDataStreamItems; dataStreamItemCounter++ )
			{
				CPLAT::CP_DataStreamItem	dataStreamItem = dataStream.GetItem( dataStreamItemCounter );

				outputString = CPLAT::CP_Utility::CP_UInt32ToString( dataStreamItemCounter );
				penLocation.Set( kLeftMargin_Flavor, penLocation.v() );
				graphics.DrawString( fontSpec, outputString, penLocation );

					// flavor
				outputString = "Flavor: ";
				penLocation.Set( kLeftMargin_Flavor + 20, penLocation.v() );
				graphics.DrawString( fontSpec, outputString, penLocation );
					// advance to 2 column on same line 
				penLocation.Set( kLeftMargin_Flavor + 20 + k2ndColumnOffset, penLocation.v() );
				flavorType = dataStreamItem.GetFlavorType().Get();
				CPLAT::CP_Utility::CP_CopyMemory( &flavorType, buffer, sizeof(UInt32) );
				buffer[ sizeof(UInt32) ] = '\0';
				outputString = "'";
				outputString += buffer;
				outputString += "'";
				graphics.DrawString( fontSpec, outputString, penLocation );
				
					// advance to next line
				penLocation.Set( kLeftMargin_Flavor + 20, penLocation.v() + kLineHeight );

					// flavor flags
				outputString = "Flags: ";
				graphics.DrawString( fontSpec, outputString, penLocation );
					// advance to 2 column on same line 
				penLocation.Set( kLeftMargin_Flavor + 20 + k2ndColumnOffset, penLocation.v() );
				graphics.DrawString( fontSpec, CPLAT::CP_Utility::CP_UInt32ToString( dataStreamItem.GetFlavorFlags() ), penLocation );

					// advance to next line
				penLocation.Set( kLeftMargin_Flavor, penLocation.v() + kLineHeight );

				CPLAT::CP_HandleBlock		*dataStreamData = dataStreamItem.GetFlavorData();
				
				if( dataStreamData != NULL )
				{
						// data size
					outputString = "Length: ";
					penLocation.Set( kLeftMargin_Flavor + 20, penLocation.v() );
					graphics.DrawString( fontSpec, outputString, penLocation );
						// advance to 2 column on same line 
					penLocation.Set( kLeftMargin_Flavor + 20 + k2ndColumnOffset, penLocation.v() );
					graphics.DrawString( fontSpec, CPLAT::CP_Utility::CP_UInt32ToString( dataStreamData->Length() ), penLocation );

						// advance to next line
					penLocation.Set( kLeftMargin_Flavor + 20, penLocation.v() + kLineHeight );

						// data
					outputString = "Data: ";
					graphics.DrawString( fontSpec, outputString, penLocation );

						// advance to 2 column on same line 
					penLocation.Set( kLeftMargin_Flavor + 20 + k2ndColumnOffset, penLocation.v() );

					char					*droppedData = dataStreamData->GetPtr();
					
					if( droppedData != NULL )
					{
						UInt32			dataSize = dataStreamData->Length();
						UInt32			dataOffset = 0;
						UInt32			x = 0;
						UInt32			y = 0;
						
						while( dataOffset < dataSize )
						{
							UInt32		lineLength	= CP_MIN( 16UL, dataSize - dataOffset );
							char		lineBuffer[ 100 ];
							
							CP_STD::strcpy(lineBuffer, "");
							
							for( x = 0; x < lineLength; x += 2)
							{
								char	byteBuffer[ 16 ];
								
								if( lineLength == x + 1)
									CP_STD::sprintf( byteBuffer, "%2.2x ",  droppedData[ dataOffset + x ] & 0x0FF );
								else
									CP_STD::sprintf( byteBuffer, "%2.2x%2.2x ", droppedData[ dataOffset + x ] & 0x0FF, droppedData[ dataOffset + x + 1 ] & 0x0FF );
								
								CP_STD::strcat( lineBuffer, byteBuffer );
							}
							
							CP_STD::sprintf( buffer, "%-41s'", lineBuffer );
							
							for( y = 0; y < lineLength; y++)
							{
								char		c = droppedData[ dataOffset + y ];
								char		cbuffer[ 16 ];
								
								if( ( c < ' ' ) || ( c > '~' ) )
									c = '.';
								
								CP_STD::sprintf( cbuffer, "%c", c );
								CP_STD::strcat(buffer, cbuffer);
							}
							
							CP_STD::strcat( buffer, "'" );
							dataOffset += lineLength;
							
							outputString = buffer;
							graphics.DrawString( fontSpec, outputString, penLocation );

								// advance to next line
							penLocation.Set( kLeftMargin_Flavor + 20 + k2ndColumnOffset, penLocation.v() + kLineHeight );
						}
					}

						// advance to next line and add additional blank line between items
					penLocation.Set( kLeftMargin_Flavor + 20, penLocation.v() + (kLineHeight * 2) );
				}
			}
		}
		
		CPLAT::CP_Size		imageSize = GetImageSize();
		CPLAT::CP_Size		newImageSize;
		
		newImageSize.Set( imageSize.Width(), penLocation.v() );
		
		if( newImageSize != imageSize )
		{
			CPLAT::CP_Rect		imageBounds( 0, 0, newImageSize.Width(), newImageSize.Height() );
			SetImageBounds( imageBounds );
		}
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Events ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	InitEventHandler
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Install the events for this view.
*	
*	@param		inEventHandler			The event handler to add our events to.
*	
*	@exception	none
*
*	@return		void
*/
void DragDropContentsView::InitEventHandler( CPLAT::CP_EventHandler& inEventHandler )
{
	#if TARGET_OS_WIN32
			// install our custom view event handlers.
			// This will install event handlers on our custom (not system defined)
			// views for handling common events.
		CPLAT::CP_EventSignal			*droppedFilesSignal = CP_new CPLAT::CP_EventSignal();
		
		droppedFilesSignal->connect( CPLAT::CP_SigSlot::slot(this, &DragDropContentsView::HandleDroppedFilesEvent) );
		inEventHandler.Add( CPLAT::kCP_WINDOW_EVENT, WM_DROPFILES, droppedFilesSignal, GetContainingWindow()->GetOSWindow() );
	#endif
}

//=========================================================================
//	FUNCTION:	HandleDroppedFilesEvent
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Handle the Windows WM_DROPFILES event. We just return CP_noErr.
*	
*	@param		inEvent					The CP_Event when a user dropped files
*										onto this view.
*	
*	@exception	none
*
*	@return		void
*/
void DragDropContentsView::HandleDroppedFilesEvent( CPLAT::CP_Event_Base& inEvent )
{
	CPLAT::OSStatus					status = CP_noErr;

	inEvent.SetStatus( status );
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
