//=========================================================================
//
//	FILE:				CP_FTP.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to implement the FTP protocol.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 22 $
//						Latest Checkout:  $Date: 2004-12-28 19:35:48 -0800 (Tue, 28 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_FTP
#define _H_CP_FTP
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_File.h"
#include "CP_FTPResponse.h"
#include "CP_NetProtoBase.h"
#include "CP_Socket.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

	// FTP commands
#define kCP_FTP_CMD__User				"USER"
#define kCP_FTP_CMD__Pass				"PASS"
#define kCP_FTP_CMD__Account			"ACCT"
#define kCP_FTP_CMD__ChangeDir			"CWD"
#define kCP_FTP_CMD__ChangeDirUp		"CDUP"
#define kCP_FTP_CMD__StructMount		"SMNT"
#define kCP_FTP_CMD__Reinit				"REIN"
#define kCP_FTP_CMD__Quit				"QUIT"

#define kCP_FTP_CMD__PortCmd			"PORT"
#define kCP_FTP_CMD__Passive			"PASV"
#define kCP_FTP_CMD__Type				"TYPE"
#define kCP_FTP_CMD__FileStruct			"STRU"
#define kCP_FTP_CMD__XferMode			"MODE"

#define kCP_FTP_CMD__Retrieve			"RETR"
#define kCP_FTP_CMD__Store				"STOR"
#define kCP_FTP_CMD__StoreUnique		"STOU"
#define kCP_FTP_CMD__Append				"APPE"
#define kCP_FTP_CMD__Allocate			"ALLO"
#define kCP_FTP_CMD__Restart			"REST"
#define kCP_FTP_CMD__RenameFrom			"RNFR"
#define kCP_FTP_CMD__RenameTo			"RNTO"
#define kCP_FTP_CMD__Abort				"ABOR"
#define kCP_FTP_CMD__Delete				"DELE"
#define kCP_FTP_CMD__RemoveDir			"RMD"
#define kCP_FTP_CMD__MakeDir			"MKD"
#define kCP_FTP_CMD__PrintDir			"PWD"
#define kCP_FTP_CMD__List				"LIST"
#define kCP_FTP_CMD__NameList			"NLST"
#define kCP_FTP_CMD__Site				"SITE"
#define kCP_FTP_CMD__System				"SYST"
#define kCP_FTP_CMD__Status				"STAT"
#define kCP_FTP_CMD__Help				"HELP"
#define kCP_FTP_CMD__NOOP				"NOOP"
#define kCP_FTP_CMD__MACB				"MACB"

#define kCP_FTP_PASSWORD__Anonymous 			"anonymous"
#define kCP_FTP_DefaultSendBufferSize 			(1024 * 4)		// use 4k default buffers for send/receiving data
#define kCP_FTP_DefaultReceiveBufferSize		(1024 * 4)

/*!
*	@class		CP_FTP
*	@brief		A CP_NetProtoBase class that implements the FTP internet protocol over a CP_Socket object.
*/
class CP_EXPORT CP_FTP : public CP_NetProtoBase
{
public:
	/*! The port number used for this class. */
	enum { kCP_FTPPortNumber = 21 };

	/*! Current state of the FTP connection with the server. */
	enum CP_E_FTPConnectionState
	{
		CP_E_FTPClosed,
		CP_E_FTPConnected
	};

	/*! Type of information being transfered. */
	enum CP_E_FTPTransferType
	{
		CP_FTP_UndefinedType = 0,
		CP_FTP_ASCIIType = 'A',
		CP_FTP_ImageType = 'I'
	};
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FTP( CP_Socket *inSocketConnection );
	//! Destructor.
	virtual								~CP_FTP() throw();
	//@}

	//! @name Connection
	//@{
				void					Connect(
												const CP_String& inAddress,
												SInt16 inPortNumber = CP_FTP::kCP_FTPPortNumber );
	//@}
				CP_E_FTPConnectionState GetConnectionState() const { return fConnectionState; }
				
																
	//! @name Commands - Connection
	//@{
				void					Disconnect();
				void					GetList(
												CP_String& inOutList,
												const CP_String& inWildcard = "" );
	//@}
												
	//! @name Commands - Directory
	//@{
				void					ChangeDirectory( const CP_String& inDirectory );
				void					ChangeDirectoryUp();
				CP_String				GetWorkingDirectory();
				void					MakeDirectory( const CP_String& inDirectory );
				void					RemoveDirectory( const CP_String& inDirectory );
	//@}

	//! @name Commands - File
	//@{
				void					DeleteFile( const CP_String& inFileName );
				void					GetFile(
												CP_File *inDestinationFile,
												const CP_String& inFileName,
												CP_E_FTPTransferType inTransferType );
				void					PutFile(
												CP_File *inSourceFile,
												const CP_String& inDestFileName,
												CP_E_FTPTransferType inTransferType,
												bool inCreateUnique );
				void					RenameFile(
												const CP_String& inOriginalName,
												const CP_String& inNewName );
				void					RenameFileFrom( const CP_String& inOriginalName );
				void					RenameFileTo( const CP_String& inNewName );
				SInt32					SendRETR( const CP_String& inFileName );
				void					SendSTOR(
												const CP_String& inFileName,
												bool inCreateUnique );
				void					SendTYPE( CP_E_FTPTransferType inTransferType );
	//@}
				
	//! @name Commands - Misc
	//@{
				void					SendPORT(
												UInt32 inHostNum,
												SInt16 inPort );
				void					SendQUIT();
				void					SendSYST();
	//@}

	//! @name Getters
	//@{
				UInt32					GetReceiveBufferSize()										{ return( fReceiveBufferSize ); }
				UInt32					GetSendBufferSize()											{ return( fSendBufferSize ); }
	//@}

	//! @name Login
	//@{
				void					LogIn(
												const CP_String& inUser,
	//@}
												const CP_String& inPassword );
												
	//! @name Misc
	//@{
				CP_Socket				*GetDataPort();
	//@}

	//! @name Setters
	//@{
				void					SetReceiveBufferSize( UInt32 inBufferSize )					{ fReceiveBufferSize = inBufferSize; }
				void					SetSendBufferSize( UInt32 inBufferSize )						{ fSendBufferSize = inBufferSize; }
	//@}

protected:
	//! @name File helper
	//@{
				void					GetFile_internal(
												CP_Socket *inSocket,
												CP_File *inDestinationFile );
				void					PutFile_internal(
												CP_Socket *inSocket,
												CP_File *inSourceFile );
	//@}

private:

	// class data
protected:
								
	UInt32						fReceiveBufferSize;		/*! Size of the buffer to hold received information.*/
								
	UInt32						fSendBufferSize;		/*! Size of the buffer to hold sent information.*/

	CP_E_FTPConnectionState		fConnectionState;		/*! Current state of the connection.*/
								
	CP_E_FTPTransferType		fTransferType;			/*! Type of data being transfered.*/
			
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FTP


