/*
=========================================================================

	FILE:		CP_Graphics.h
	
				Copyright  2002-2003-2003. Ken Stahlman. All Rights Reserved.
	
	PURPOSE:	A class to handle drawing into a CP_Canvas object. 
	
	HISTORY:	1st Version - January 2002.
	
	NOTES:		- none -

	VERSION:	Current Revision: $Revision: 232 $
				Latest Checkout:  $Date: 2005-07-14 22:13:17 -0700 (Thu, 14 Jul 2005) $

=========================================================================
*/
#ifndef _H_CP_Graphics
#define _H_CP_Graphics
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_FontSpec.h"
#include "CP_Graphics_Enums.h"
#include "CP_Graphics_Imp.h"
#include "CP_NoCopy.h"
#include "CP_Rect.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_Graphics : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Graphics(
												CP_Canvas& inDestCanvas,
												bool inSynchronizeWithPort = true,
												CP_E_GraphicsImp inImpClassType = CP_E_GraphicsImp_PlatformNative );
	//! Destructor.
	virtual								~CP_Graphics();
	//@}

	//! @name Implementor
	//@{
										/*!
										*	Returns implementation object that is doing the actual work of this class.
										*	
										*	@exception	none
										*	
										*	@return		CP_Graphics_Imp			The object implementing this class.
										*/
			CP_Graphics_Imp				*GetImpClass()											{ return( fImpClass ); }

										/*!
										*	Sets the implementation object that is doing the actual work of this class.
										*
										*	@param		inImpClass				The object implementing this class.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetImpClass( CP_Graphics_Imp *inImpClass )				{ fImpClass = inImpClass; }
	//@}

	//! @name Getters
	//@{
			float						GetLineWidth();
			float						GetAlpha();
			CP_ARGBColor				GetStrokeColor();
			CP_ARGBColor				GetFillColor();
	//@}

	//! @name State
	//@{
			void						SaveState();
			void						RestoreState();
			
			void						ScaleCTM(
											float inHorAmt,
											float inVerAmt );
			void						RotateCTM( float inAngle );
			void						SkewCTM(
											float inHorAmt,
	//@}
											float inVerAmt );

	//! @name Arcs
	//@{
			void						StrokeArc(
												float inHorCenter,
												float inVerCenter,
												float inRadius,
												float inStartAngle,
												float inEndAngle );
			void						FillArc(
												float inHorCenter,
												float inVerCenter,
												float inRadius,
												float inStartAngle,
												float inEndAngle );
	//@}

	//! @name Bezier Curves
	//@{
			void						StrokeBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inControlPointTwo,
												const CP_Point& inEndPoint );
			void						StrokeQuadBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inEndPoint );
	//@}

	//! @name Gradient
	//@{
			void						DrawGradient(
												CP_E_GradientType inGradietType,
												CP_Rect& inDestBounds );
			void						SetGradientAngle( SInt16 inAngle );
			void						SetGradientColor(
												const SInt16 inIndex,
												const CP_ARGBColor& inColour );
			void						SetGradientCount( SInt16 inGradientCount );
	//@}
			
	//! @name Lines
	//@{
										/*!
										*	Draws a line from inStartPoint to inEndPoint
										*
										*	@param		inStartPoint			Starting point to draw the line.
										*	@param		inEndPoint				Ending point for the line.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						Line(
												const CP_Point& inStartPoint,
												const CP_Point& inEndPoint )					{ Line( inStartPoint.h(), inStartPoint.v(), inEndPoint.h(), inEndPoint.v() ); }
			void						Line(
												float inStartHorLocation,
												float inStartVerLocation,
												float inEndHorLocation,
												float inEndVerLocation );
	//@}

	//! @name Icons
	//@{
			void						DrawIcon(
												const CP_Rect& inBounds,
												CP_E_IconStyle inIconType,
												SInt16 inIconID );
			void						DrawIcon(
												CP_OSIcon& inPicture,
												const CP_Rect& inBounds );
	//@}

	//! @name Images
	//@{
			void						DrawImage(
												SInt16 inImageResID,
												CP_Rect& inDestBounds );
			void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inDestBounds );
			void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inSrcBounds,
												CP_Rect& inDestBounds );
			void						DrawPicture(
												CP_OSPicture& inPicture,
												const CP_Rect& inBounds );
	//@}
			
	//! @name Paths
	//@{
			void						BeginPath();
			void						ClosePath();
			void						StrokePath();
			void						FillPath();
			void						FillPath( CP_Pattern& inPattern );
			void						DrawPath( CP_E_PathDrawingMode inPathDrawMode );
			void						PathMoveTo( const CP_Point& inPoint );
			void						PathMoveTo(
												float inHorLocation,
												float inVerLocation );
			void						PathAddLineTo( const CP_Point& inPoint );
			void						PathAddLine(
												const CP_Point& inStartPoint,
												const CP_Point& inEndPoint );
			void						PathAddLine(
												float inStartH,
												float inStartV,
												float inEndH,
												float inEndV );
			void						PathAddLines(
												const CP_Point inPoint[],
												SInt16 inNumberPoints );
			void						PathAddRect( const CP_Rect& inBounds );
			void						PathAddOval( const CP_Rect& inBounds );
			void						PathAddBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inControlPointTwo,
												const CP_Point& inEndPoint );
			void						PathAddQuadBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inEndPoint );
			void						PathAddPoly( CP_Point_Array& inPoints );
			void						PathAddArc(
												float inHorCenter,
												float inVerCenter,
												float inRadius,
												float inStartAngle,
												float inEndAngle,
												SInt16 inClockwise );
			void						ClipPath();
	//@}

	//! @name Polygons
	//@{
			void						StrokePoly( CP_Point_Array& inPoints );
			void						FillPoly( CP_Point_Array& inPoints );
	//@}
									
	//! @name Rects
	//@{
			void						StrokeRect( const CP_Rect& inBounds );
			void						EraseRect( const CP_Rect& inBounds );
			void						FillRect( const CP_Rect& inBounds );
	//@}
			
	//! @name Regions
	//@{
			void						StrokeRegion( const CP_Region& inRegion );
			void						FillRegion( const CP_Region& inRegion );
	//@}

	//! @name Round Rects
	//@{
			void						StrokeRoundRect(
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
			void						FillRoundRect( 
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	//@}

	//! @name Oval / Circle
	//@{
			void						StrokeOval( const CP_Rect& inBounds );
			void						FillOval( const CP_Rect& inBounds );
	//@}

	//! @name Setters
	//@{
			void						SetAlpha( float inAlpha );

			void						SetLineWidth( float lineWidth );
			void						SetLineJoin( CP_E_GraphicsLineJoin inLineJoin );
			void						SetLineCap( CP_E_GraphicsLineCap inLineCap );

			void						SetStrokeColor( CP_ARGBColor& inStrokeColor );
			void						SetFillColor( CP_ARGBColor& inFillColor );
	//@}
			
	//! @name Text
	//@{
			void						SetTextRenderingFlags( SInt16 inFlags );
			void						DrawString(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Point& inStartPoint );
			void						DrawText(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop,
												bool inWrapText = true );
			void						DrawTrunc(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop );
	//@}

protected:

	// class data
protected:
	CP_Graphics_Imp						*fImpClass;				/*! The object that is implementing these functions. */
	CP_E_GraphicsImp					fImpClassType;			/*! The type of implementation to be used. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Graphics
