//=========================================================================
//
//	FILE:				CP_InternetMessage.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class to handle internet style messages (RFC822) used
//						in sending and receiving information to servers that implement
//						various internet protocols.
//
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_InternetMessage
#define _H_CP_InternetMessage
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_InternetMessage
*	@brief		Base class to handle internet style messages (RFC822) used in sending and
*				receiving information to servers that implement various internet protocols.
*/
class CP_EXPORT CP_InternetMessage
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_InternetMessage();
	//! Destructor.
	virtual								~CP_InternetMessage() throw();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Return the body section of the message.
										*
										*	@exception	none
										*
										*	@return		CP_String const		The body of the message.
										*/
				CP_String				GetBody() const									{ return( fBody ); }

										/*!
										*	Return the header section of the message.
										*
										*	@exception	none
										*
										*	@return		CP_String const		The header of the message.
										*/
				CP_String				GetHeader() const								{ return( fHeader ); }
	//@}

	//! @name Header
	//@{
	virtual		void					AddHeaderField(
											const CP_String& inHeaderTag,
											const CP_String& inHeaderBody,
											const CP_String& inHeaderLineTermination = CP_CRLF );
	//@}
											
	//! @name Setters
	//@{
										/*!
										*	Sets the body section of the message.
										*
										*	@param		inBody			The body of the message.
										*
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetBody( const CP_String& inBody )				{ fBody = inBody; }

										/*!
										*	Sets the header section of the message.
										*
										*	@param		inHeader			The header of the message.
										*
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetHeader( const CP_String& inHeader )			{ fHeader = inHeader; }

										/*!
										*	Set the flag to indicate if this is a message we are receiving,
										*	or sending. If inIsIncoming is true, then this is a received message.
										*
										*	@param		inIsIncoming		Flag to indicate if incoming message.
										*
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetIsIncomingMessage( bool inIsIncoming )		{ fIsIncomingMessage = inIsIncoming; }

										/*!
										*	Set a flag to indicate if the header is formatted for sending.
										*
										*	@param		inIsFormatted		Flag to indicate if the message is formatted for sending.
										*
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetIsHeaderFormatted( bool inIsFormatted )		{ fHeaderIsFormatted = inIsFormatted; }
	//@}

	//! @name Testers
	//@{
										/*!
										*	Returns the IsIncomingMessage flag.
										*
										*	@exception	none
										*
										*	@return		bool		True if the message is incoming.
										*/
				bool					IsIncomingMessage()								{ return( fIsIncomingMessage ); }

										/*!
										*	Returns the flag to indicate if the message is formatted for sending.
										*
										*	@exception	none
										*
										*	@return		bool		True if the message is formatted.
										*/
				bool					IsHeaderFormatted()								{ return( fHeaderIsFormatted ); }
	//@}

protected:

private:
	//! @name Initialization
	//@{
				void					Init();
	//@}
										
	// Data members
protected:
	CP_String							fBody;					/*! The body of the message in a CP_String object. */
	CP_String							fHeader;				/*! The header of the message in a CP_String object. */
	bool								fIsIncomingMessage;		/*! Flag to indicate if this is an incoming message. */
	bool								fHeaderIsFormatted;		/*! Flag to indicate if the header is currently formatted. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_InternetMessage


