//=========================================================================
//
//	FILE:				CP_M_ReferenceCounted.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class for objects that are reference counted.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 225 $
//						Latest Checkout:  $Date: 2005-07-01 17:21:21 -0700 (Fri, 01 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_M_RefCounted
#define _H_CP_M_RefCounted
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_M_RefCounted
*	@brief		Mix-in class for objects that are reference counted.
*/

class CP_EXPORT CP_M_RefCounted
{
public:
	//! @name Reference
	//@{
			UInt32						AddReference() const;
			UInt32						RemoveReference() const;
	//@}
			
	//! @name Autorelease
	//@{
			void 						Autorelease() const;
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the current ref count.
										*	
										*	@exception	none
										*	
										*	@return		UInt32			The current referencce count.
										*/
			UInt32						GetRefCount() const									{ return( fRefCount ); }
	//@}

	//! @name operators
	//@{
			CP_M_RefCounted&			operator = ( const CP_M_RefCounted& inRefCounted );
	//@}
			
protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_RefCounted();
										CP_M_RefCounted( CP_M_RefCounted& inRefCounted );
	//! Destructor.
	virtual								~CP_M_RefCounted();
	//@}

	// class data
private:
	mutable UInt32						fRefCount;	/*! The number of references to this object. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

template <class T> 
inline const T* CP_Autorelease(const T* inObj)
{
	if( inObj )
		inObj->Autorelease();
	
	return( inObj );
}

template <class T> 
inline T* CP_Autorelease(T* inObj)
{
	if( inObj )
		inObj->Autorelease();
	
	return( inObj );
}

/*!
*	@class  CP_SmartRef
*	@brief	Template class representing a smart pointer for reference counted objects.
*/
template <class T> 
class CP_SmartRef
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_SmartRef()
											: fObj(NULL)									{}
										CP_SmartRef(
												T *inObj,
												bool inTransferOwnership=false )
													: fObj(inObj)							{ if( !inTransferOwnership && fObj ) fObj->AddReference(); }
										CP_SmartRef( const CP_SmartRef<T>& inRef )
											: fObj(inRef.fObj)								{ if( fObj ) fObj->AddReference(); }
										template <class Other>
										CP_SmartRef( const CP_SmartRef<Other>& inRef )
											: fObj(NULL)									{ T* other = inRef.Get(); this->Reset( other ); } // assignment to local variable should prevent upcasts and cross-casts
	//! Destructor.
										~CP_SmartRef()										{ if( fObj ) fObj->RemoveReference(); }
	//@}
	
	//! @name operators
	//@{
			T&							operator *() const									{ return( *fObj ); }
			T*							operator->() const									{ return( fObj ); }
	
										operator T *() const								{ return( fObj ); }
	
			CP_SmartRef<T>& 			operator = ( const CP_SmartRef<T>& inRef )			{ this->Reset( inRef.fObj ); return *this; }
			template <class Other>
			CP_SmartRef<T>& 			operator = ( const CP_SmartRef<Other>& inRef )		{ this->Reset( inRef.Get() ); return *this; }
			CP_SmartRef<T>& 			operator = ( T* inObj )								{ this->Reset( inObj ); return *this; }
			template <class Other>
			CP_SmartRef<T>& 			operator = ( Other* inObj )							{ this->Reset( inObj ); return *this; }
	//@}
	
	//! @name Object management
	//@{
			T							*Get() const										{ return( fObj ); }
			T							*Reset(
												T *inObj,
												bool inTransferOwnership = false );
			T							*Release();
	//@}
	
		
	// class data
protected:
		T								*fObj;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif
	
#if TARGET_OS_WIN32
#endif
	
#if TARGET_OS_LINUX
#endif
};

template <class T>
T* CP_SmartRef<T>::Reset( T *inObj, bool inTransferOwnership )
{ 
	if ( inObj != fObj ) 
	{
		if( fObj )
			fObj->RemoveReference();
		
		fObj = inObj; 
		
		if( inObj && !inTransferOwnership )
			inObj->AddReference(); 
	}
	else if( inObj && inTransferOwnership )
	{
		inObj->RemoveReference();
	}
	
	return( fObj ); 
}

template <class T>
T* CP_SmartRef<T>::Release()
{ 
	T			*tmp = fObj;
	
	fObj = NULL; 
	
	return( tmp ); 
}

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_RefCounted


