/*
=========================================================================

	FILE:		CP_PE_Resources.h
	
	PURPOSE:	Classes to handle Windows PE file resources.
	
	HISTORY:	1st Version - October 2001.
	
	NOTES:		This is a Windows only set of classes for dealing with resources
				stored in a Window PE type file (.exe, .dll).

				This code is from:
					Matt Pietrek
					Microsoft Systems Journal, January 1998
					Program: DlgDump.EXE

	VERSION:	Current Revision: $Revision: 2 $
				Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $

=========================================================================
*/
#ifndef _H_CP_PE_Resources
#define _H_CP_PE_Resources
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32

	// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

class CP_PE_Resources;
class CP_PE_ResourceType;

//=============================================================================
// CP_PE_Resource - Contains information about a single instance of a one resources
//		type in the file.
//=============================================================================

class CP_PE_Resource
{
public:
										CP_PE_Resource(
												DWORD rsrcDataRVA,
												DWORD rsrcStartRVA,
                								DWORD loadAddress,
                								DWORD fileDelta,
                								unsigned rsrcTblIdx,
                								DWORD IdOrNameOffset );
										~CP_PE_Resource()							{}

			DWORD						Id()										{ return( m_IdOrNameOffset ); }
			BOOL						IsNamed()									{ return( (BOOL)(m_IdOrNameOffset & 0x80000000) ); }
			BOOL						Name(
												PSTR pszBuffer,
												unsigned cbBuffer );
    
			DWORD						Size()										{ return( m_pRsrcDataEntry->Size ); }
			DWORD						CodePage()									{ return( m_pRsrcDataEntry->CodePage ); }          

			PVOID						GetData();

	// class data
private:
    friend class CP_PE_ResourceType;

	DWORD							m_rsrcDataRVA;
	DWORD							m_rsrcStartRVA;
	DWORD							m_loadAddress;
	DWORD							m_fileDelta;
	unsigned						m_rsrcTblIdx;
	DWORD							m_IdOrNameOffset;
    
	PIMAGE_RESOURCE_DATA_ENTRY	m_pRsrcDataEntry;
};

//=============================================================================
// CP_PE_ResourceDirectory - Class that wraps the IMAGE_RESOURCE_DIRECTORY structure
//		defined in WINNT.H
//=============================================================================
class CP_PE_ResourceDirectory
{
protected:
											CP_PE_ResourceDirectory(
													DWORD rsrcDirRVA,
													DWORD rsrcStartRVA,
													DWORD loadAddress,
													DWORD fileDelta );
											~CP_PE_ResourceDirectory( )						{};

			void							InitPointerMembers();
            
			DWORD							Characteristics()								{ return( m_pRsrcDirHdr->Characteristics ); }
			DWORD							TimeDateStamp()									{ return( m_pRsrcDirHdr->TimeDateStamp ); }
			WORD							MajorVersion()									{ return( m_pRsrcDirHdr->MajorVersion ); }
			WORD							MinorVersion()									{ return( m_pRsrcDirHdr->MinorVersion ); }
			WORD							NumberOfNamedEntries()							{ return( m_pRsrcDirHdr->NumberOfNamedEntries ); }
			WORD							NumberOfIdEntries()								{ return( m_pRsrcDirHdr->NumberOfIdEntries ); }

			DWORD							NumberOfEntries()								{ return( m_nEntries ); }
    
			PIMAGE_RESOURCE_DIRECTORY_ENTRY	GetResourceEntryByName( LPCTSTR pszName );
			PIMAGE_RESOURCE_DIRECTORY_ENTRY	GetResourceEntryById( WORD Id );

	// class data
protected:
    DWORD							m_rsrcDirRVA;
    DWORD							m_rsrcStartRVA;
    DWORD							m_loadAddress;
    DWORD							m_fileDelta;
    DWORD							m_nEntries;

    PIMAGE_RESOURCE_DIRECTORY		m_pRsrcDirHdr;
    PIMAGE_RESOURCE_DIRECTORY_ENTRY	m_pRsrcDirEntries;
    PIMAGE_RESOURCE_DIRECTORY_ENTRY	m_pRsrcIdDirEntries;
};

//=============================================================================
// CP_PE_Resources represents one particular type of resource in an executable
// (for instance, BITMAPs, or DIALOGs)
//=============================================================================

//=============================================================================
// CP_PE_ResourceType - Class that wraps one particular type of resources in a
//		file, ex. BITMAP, ICON, etc
//=============================================================================
class CP_PE_ResourceType : public CP_PE_ResourceDirectory
{
public:
											~CP_PE_ResourceType()						{}

			CP_PE_Resource					*GetNextResourceInst( CP_PE_Resource *pPrev );
			CP_PE_Resource					*GetResourceInstByName( PSTR pszName );
			CP_PE_Resource					*GetResourceInstById( WORD Id );
    
			DWORD							Id()										{ return( m_IdOrNameOffset ); }
			BOOL							IsNamed()									{ return( (BOOL)(m_IdOrNameOffset & 0x80000000) ); }
			BOOL							Name(
													PSTR pszBuffer,
													unsigned cbBuffer );

private:			

    friend class CP_PE_Resources;

											CP_PE_ResourceType(
													DWORD rsrcDirRVA,
													DWORD rsrcStartRVA,
													DWORD loadAddress,
													DWORD fileDelta,
													unsigned rsrcTblIdx,
													DWORD IdOrNameOffset );
                
    
			DWORD							BurrowDownOneDirLevel( DWORD offsetToDir );

	// class data
private:
	unsigned	m_rsrcTblIdx;
	DWORD		m_IdOrNameOffset;
};

//=============================================================================
// CP_PE_Resources represents the entire collection of resources in an executable
//=============================================================================

class CP_PE_Resources : public CP_PE_ResourceDirectory
{
public:
											CP_PE_Resources( LPCTSTR pszFilename );
											~CP_PE_Resources( );
    
			CP_PE_ResourceType				*GetNextResourceType( CP_PE_ResourceType *pPrev );
			CP_PE_ResourceType				*GetResourceTypeByName( LPCTSTR pszName );
			CP_PE_ResourceType				*GetResourceTypeById( WORD Id );
    
	// class data
private:
	HANDLE		m_hFile;
	HANDLE		m_hFileMapping;
	PVOID		m_pMemoryMappedFileBase;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_WIN32

#endif	//_H_CP_PE_Resources


