//=========================================================================
//
//	FILE:				CP_POP3.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to implement the POP3 protocol.
//
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 162 $
//						Latest Checkout:  $Date: 2005-04-28 15:05:07 -0700 (Thu, 28 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_POP3
#define _H_CP_POP3
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_MailMessage.h"
#include "CP_NetProtoBase.h"
#include "CP_POP3Response.h"
#include "CP_Socket.h"
#include "CP_STLArrayDefines.h"
#include "CP_String.h"
#include "CP_TimedTask.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

	// POP3 commands
#define kCP_POP3_CMD__User				"USER"
#define kCP_POP3_CMD__Pass				"PASS"
#define kCP_POP3_CMD__Stat				"STAT"
#define kCP_POP3_CMD__List				"LIST"
#define kCP_POP3_CMD__Retr				"RETR"
#define kCP_POP3_CMD__Dele				"DELE"
#define kCP_POP3_CMD__NOOP				"NOOP"
#define kCP_POP3_CMD__Rset				"RSET"
#define kCP_POP3_CMD__UIDL				"UIDL"
#define kCP_POP3_CMD__Top				"TOP"
#define kCP_POP3_CMD__APOP				"APOP"
#define kCP_POP3_CMD__Quit				"QUIT"

const UInt32 kCP_TicklePeriod							= 60;

/*!
*	@class		CP_POP3
*	@brief		A CP_NetProtoBase class to handle the POP3 internet protocol for receiving messages.
*/
class CP_EXPORT CP_POP3 : public CP_NetProtoBase, public CP_SigSlot::CP_SlotObject
{
public:
		// default POP3 port number
	enum { kCP_POP3PortNumber = 110 };

	enum POP3ConnectState
	{
		kCP_POP3Closed,
		kCP_POP3Connected
	};

	enum POP3ServerState
	{
		kCP_POP3Offline,
		kCP_POP3Authorization,
		kCP_POP3Transaction,
		kCP_POP3Update
	};

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_POP3( CP_Socket *inSocketConnection );
	//! Destructor.
	virtual								~CP_POP3() throw();
	//@}

	//! @name Connection
	//@{
			void						Connect(
												const CP_String& inUser,
												const CP_String& inPassword,
												const CP_String& inAddress,
												SInt16 inPortNumber = CP_POP3::kCP_POP3PortNumber );
			void						Disconnect();
								
			void						WaitForServerID();
	//@}

	//! @name Commands
	//@{
			void						SendLIST();
			void						SendNOOP();
			void						SendQUIT();
			void						SendSTAT(
												UInt32 *inOutNumberMails,
												UInt32 *inOutTotalMailSize );
			void						SendTOP(
												UInt32 inMessageID,
												UInt32 inMessageLines,
												CP_MailMessage& inOutMessage );
			void						SendUIDL();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the connection state to the POP3 server.
										*	
										*	@exception	none
										*
										*	@return		POP3ConnecctState		Connection state.
										*/
			POP3ConnectState			GetConnectionState()							{ return( fConnectState ); }

										/*!
										*	Returns the state to the POP3 server.
										*	
										*	@exception	none
										*
										*	@return		POP3ServerState			Server state.
										*/
			POP3ServerState				GetServerState()								{ return( fServerState ); }

			UInt32						GetMessageSize( UInt32 inMessageIndex );
			UInt32 						GetMessageID( UInt32 inMessageIndex );

			void						GetAllHeaders( CP_MailMessage_Array& inMailMessages );

			void						GetMailMessage(
												UInt32 inWhichMessage,
												CP_MailMessage& inOutMessage );
	//@}

protected:
			void						HandleIdle(
												CP_DataValueHolder& inDataValue,
												OSStatus& inOSStatus );

	//! @name Sending
	//@{
			void						SendAuthentication(
												const CP_String& inUser,
												const CP_String& inPassword );
	
			void						SendCommandWithMultiLineReply(
												CP_String& inCommand,
												CP_InternetProtocolProgress& inProtocolProgress );
	//@}

private:

	// class data
protected:
	POP3ConnectState					fConnectState;			/*! Connection state to the POP3 server. */
	POP3ServerState						fServerState;			/*! POP3 server state. */	

	UInt32								fLastTickleTicks;		/*! Last time we communicated with the server. */

	CP_UInt32s_Array					fMessageSizes;			/*! List of the size of all messages on the server. */
	CP_UInt32s_Array					fMessageIDs;			/*! List of all messages ids on the server. */

	CP_TimedTask						*fIdleTimer;			/*! Timer used to tickle server every so often. */
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_POP3


