//=========================================================================
//
//	FILE:				CP_PropertyList_Array.h
//	
//						Created by Jens Miltner on 01.12.04.
//	
//	PURPOSE:			CP_PropertyList arrays.
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 115 $
//						Latest Checkout:  $Date: 2005-03-24 16:55:37 -0800 (Thu, 24 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_PropertyList_Array
#define _H_CP_PropertyList_Array

#include "CP_PropertyList_Node.h"
#include <iterator>
#include <utility>
#if CP_PropertyList_Uses_CoreFoundation
	#include "CP_CFMutableObject.h"
#endif

CPLAT_Begin_Namespace_CPLAT

class CP_Date; 
class CP_Time;
class CP_DateTime;
class CP_UUID;

namespace CP_PropertyList {

	class CP_EXPORT Array : public Node
#if CP_PropertyList_Uses_CoreFoundation
		, protected CP_CFMutableObject<CFArrayRef, CFMutableArrayRef>
#endif
	{
	public:
		enum { 
			npos = 0xffffffff	//!< specifies a position beyond the end of the array
		};

	#if CP_PropertyList_Uses_CoreFoundation		
		class _reference;
		class _iterator;
		class _const_iterator;
		
		typedef UInt32						size_type;
		typedef UInt32						difference_type;
		typedef Node*						value_type;
		typedef const Node*					const_reference;
		typedef _reference					reference;
		typedef _iterator					iterator;
		typedef _const_iterator				const_iterator;
		typedef std::reverse_iterator<iterator>				reverse_iterator;
		typedef std::reverse_iterator<const_iterator>		const_reverse_iterator;
	#else
		typedef std::vector<CP_SmartRef<const Node> > ElementsVector;
		
		typedef ElementsVector::size_type								size_type;
		typedef ElementsVector::difference_type					difference_type;
		typedef ElementsVector::value_type								value_type;
		typedef ElementsVector::const_reference					const_reference;
		typedef ElementsVector::reference							reference;
		typedef ElementsVector::iterator									iterator;
		typedef ElementsVector::const_iterator						const_iterator;
		typedef ElementsVector::reverse_iterator					reverse_iterator;
		typedef ElementsVector::const_reverse_iterator		const_reverse_iterator;
	#endif

		Array();
		//explicit Array(const Node** inValues);
		Array(const Array& inSrc);
		explicit Array(const std::vector<const Node*>& inValues);
	#if CP_PropertyList_Uses_CoreFoundation
		explicit Array(CFArrayRef inArrayRef, bool inRetain=true);
		explicit Array(const std::vector<CFTypeRef>& inValues);
	#endif
		virtual ~Array() throw();
		
		template <class T> struct _identity { const T& operator()(const T& value) const { return value; } };
		template <class _InputIterator, class _ValueConverter>
		static Array* ArrayFromValues(_InputIterator begin, _InputIterator end, _ValueConverter converter=_identity<typename _InputIterator::value_type>())
		{
			std::vector<const Node*>	nodes(std::distance(begin, end));
			std::transform(begin, end, nodes.begin(), converter);
			return CP_Autorelease(CP_new Array(nodes));
		}
	#if CP_PropertyList_Uses_CoreFoundation
		template <class _InputIterator, class _ValueConverter>
		static Array* ArrayFromCFObjects(_InputIterator begin, _InputIterator end, _ValueConverter converter=_identity<typename _InputIterator::value_type>())
		{
			std::vector<CFTypeRef>	nodes(std::distance(begin, end));
			std::transform(begin, end, nodes.begin(), converter);
			return CP_Autorelease(CP_new Array(nodes));
		}
	#endif
		
		//! @name Getting / Setting values
		//! @{
		/*!
			@brief   Get the value at the specified position.
		 
			Extracts the value at the specified index and returns the corresponding \ref CP_PropertyList::Node instance.
			 
			@exception CP_BoundsException		raised if index is not in the legal range [0..n[,
												where n is the array's element count
			 
			@param     inIndex		The index of the value to retrieve. 
			 
			@return    An object representing the value at the specified index.
		*/
		const Node* GetValue(size_type inIndex) const;
		/*!
			@brief   Set a value in the array.
		 
			 Sets a value in the array to the object specified by the \a inValue parameter.
			 
			 @exception CP_BoundsException		raised if index is not in the legal range [0..n],
												where n is the array's element count (or \ref npos).
			 
			 @param     inIndex		Index [0-based] of the value to set. Pass \ref npos or the current element count to append the value.
			 @param     inValue		The value to set at the specified position.
		*/
		void SetValue(size_type inIndex, const Node& inValue);
		/*!
			@brief   Append a value to the end of the array.
		 
			Appends the value specified in \a inValue to the end of the array.\n
			Semantically equivalent to <tt>SetValue(Array::npos, inValue)</tt>.
		 
			@param     inValue		The value to append.
		 */
		void AppendValue(const Node& inValue);
		/*!
			 @brief   Delete a value from the array.
		  
			 Deletes the value at the index specified by \a inIndex from the array. Elements following the
			 deleted element will be moved one position further to the front of the array.
		  
			 @exception CP_BoundsException		raised if index is not in the legal range [0..n[ 
												where n is the array's element count.

			 @param     inIndex		The position of the element to delete.
		*/
		void DeleteValue(size_type inIndex);
		
		//! @name Convenience Getters/Setters
		//! @{
		//@{
		/*!
			 @brief   Get typed value.
		  
			 Extracts the value at the specified index and returns as a native C++ type.
		  
			 @exception CP_BoundsException		raised if index is not in the legal range [0..n[,
												where n is the array's element count
			 @exception std::bad_cast			raised if value is not of the expected class.

			 @param     inIndex		The index of the value to retrieve. 
		  
			 @return    The value at the specified index.
		*/
		CP_String GetStringValue(size_type inIndex) const;
		UInt32 GetUInt32Value(size_type inIndex) const;
		SInt32 GetSInt32Value(size_type inIndex) const;
		double GetFloatValue(size_type inIndex) const;
		bool GetBooleanValue(size_type inIndex) const;
		CP_DateTime GetDateTimeValue(size_type inIndex) const;
		void GetDateTimeValue(size_type inIndex, CP_Date* outDate, CP_Time* outTime) const;
		CP_UUID GetUUIDValue(size_type inIndex) const;
		const Array* GetArrayValue(size_type inIndex) const;
		const Dictionary* GetDictionaryValue(size_type inIndex) const;
		//@}
		//@{
		/*!
			 @brief   Set a value in the array.
		  
			 Sets a value in the array to the data specified by the \a inValue parameter.
		  
			 @exception CP_BoundsException		raised if index is not in the legal range [0..n] 
												where n is the array's element count.

			 @param     inIndex		Index [0-based] of the value to set. Pass \ref npos or the current element count to append the value.
			 @param     inValue		The value to set at the specified position.
		 */
		void SetStringValue(size_type inIndex, const CP_String& inValue);
		void SetNumberValue(size_type inIndex, UInt32 inValue);
		void SetNumberValue(size_type inIndex, double inValue);
		void SetBooleanValue(size_type inIndex, bool inValue);
		void SetDateTimeValue(size_type inIndex, const CP_DateTime& inDateTime);
		void SetDateTimeValue(size_type inIndex, const CP_Date& inDate);
		void SetDateTimeValue(size_type inIndex, const CP_Time& inTime);
		void SetUUIDValue(size_type inIndex, const CP_UUID& inUUID);
		//@}
		//@{
		/*!
			 @brief   Append a value to the end of the array.
		  
			 Appends the value specified in \a inValue to the end of the array.\n
			 Semantically equivalent to <tt>SetValue(Array::npos, inValue)</tt>.
		  
			 @param     inValue		The value to append.
		 */
		void AppendStringValue(const CP_String& inValue);
		void AppendNumberValue(UInt32 inValue);
		void AppendNumberValue(double inValue);
		void AppendBooleanValue(bool inValue);
		void AppendDateTimeValue(const CP_DateTime& inDateTime);
		void AppendDateTimeValue(const CP_Date& inDate);
		void AppendDateTimeValue(const CP_Time& inTime);
		void AppendUUIDValue(const CP_UUID& inUUID);
		//@}
		//! @}
		
		//! @}
		
		//! Make a mutable array with the same contents.
		Array* MakeMutable() const;
		
		//! @name STL sequence container like interface
		//! @{
		size_type size() const;
		void resize(size_type inNewSize);
		bool empty() const;
		const Node* operator [](size_type inIndex) const;
		reference operator [](size_type inIndex);
		const Node* front() const;
		reference front();
		const Node* back() const;
		reference back();
		iterator begin();
		iterator end();
		const_iterator begin() const;
		const_iterator end() const;
		reverse_iterator rbegin();
		reverse_iterator rend();
		const_reverse_iterator rbegin() const;
		const_reverse_iterator rend() const;
		//! @}
		
		//! @name Node overrides and implementation of abstract methods
		//! @{
		virtual NodeType_t NodeType() const throw();
		static Array* FromXMLData(CP_InStream& inXMLDataStream);
		//! @}
		
		virtual void PrintToLog(CP_LogFile& inLogFile, SInt32 inLevel=0, SInt32 inIndentLevel = 0) const;
	#if CP_PropertyList_Uses_CoreFoundation
		virtual CFTypeRef/*CFArrayRef*/ GetNodeCFRef() const;
	#endif
	#if !CP_PropertyList_Uses_CoreFoundation
	protected:
		//! write XML data for this node only (i.e. without xml header, plist root, etc.)
		virtual void WriteNodeXMLData(CP_OutStream& inXMLDataStream, int inLeadingSpaces) const;
	#endif
		// class data:
	protected:
	#if !CP_PropertyList_Uses_CoreFoundation
		ElementsVector fArrayElements;
	#endif
	};

#if CP_PropertyList_Uses_CoreFoundation	
	
	// Array reference implemenation
	class Array::_reference
	{
		friend class Array;
		Array&				fArray;
		Array::size_type	fIndex;
	protected:
		_reference(Array& inArray, Array::size_type inIndex) : fArray(inArray), fIndex(inIndex) {}
	public:	
		_reference(const _reference& inSrc) : fArray(inSrc.fArray), fIndex(inSrc.fIndex) {}
		
		inline const Node* operator ->() const 
		{ 
			return fArray.GetValue(fIndex); 
		}
		inline operator const Node*() const 
		{ 
			return fArray.GetValue(fIndex); 
		}
		inline _reference& operator = (const Node* inNewValue)
		{
			ASSERT( (inNewValue != NULL) );
			fArray.SetValue(fIndex, *inNewValue);
			return *this;
		}
		inline _reference& operator = (const _reference& inNewValue)
		{ 
			return operator = (static_cast<const Node*>(inNewValue)); 
		}
	};
	
	class Array::_iterator : public std::iterator<std::random_access_iterator_tag, Node*, Array::difference_type, Node**, Array::reference>
	{
		friend class Array;
		friend class Array::_const_iterator;
		
		Array*				fArray;
		Array::size_type	fCursor;
		Array::size_type	fMaxCursor;
		enum { npos = Array::npos };
	protected:
		_iterator(Array* inArray, Array::size_type inIndex, Array::size_type maxIndex) : fArray(inArray), fCursor(inIndex), fMaxCursor(maxIndex) {}
		_iterator(Array* inArray, Array::size_type inIndex) : fArray(inArray), fCursor(inIndex), fMaxCursor(inArray->size()) {}
	public:
		typedef Array::difference_type difference_type;
		
		_iterator() : fArray(NULL), fCursor(npos), fMaxCursor(0) {}
		_iterator(const _iterator& src) : fArray(src.fArray), fCursor(src.fCursor), fMaxCursor(src.fMaxCursor) {}
		
		bool operator == (const _iterator& rhs) const { return fCursor == rhs.fCursor  &&  fArray == rhs.fArray; }
		bool operator != (const _iterator& rhs) const { return fCursor != rhs.fCursor  ||  fArray != rhs.fArray; }
		bool operator < (const _iterator& rhs) const { return fCursor < rhs.fCursor; }
		Array::reference operator *() const { return (*fArray)[fCursor]; }
		const Node* operator ->() const { return fArray->GetValue(fCursor); }
		
		_iterator& operator ++(); 
		_iterator operator ++(int);
		_iterator& operator --();
		_iterator operator --(int);

		difference_type operator - ( const _iterator& rhs ) const;
		_iterator operator + (difference_type dist) const;
		_iterator& operator += (difference_type dist);
		_iterator& operator -= (difference_type dist);
	};

	class Array::_const_iterator : public std::iterator<std::random_access_iterator_tag, const Node*, Array::difference_type, const Node**, const Array::reference>
	{
		friend class Array;
		
		Array::_iterator	fIterator;
		enum { npos = Array::_iterator::npos };
	protected:
		_const_iterator(const Array* inArray, Array::size_type inIndex, Array::size_type maxIndex) : fIterator(const_cast<Array*>(inArray), inIndex, maxIndex) {}
		_const_iterator(const Array* inArray, Array::size_type inIndex) : fIterator(const_cast<Array*>(inArray), inIndex) {}
	public:
		typedef Array::_iterator::difference_type difference_type;
		
		_const_iterator() {}
		_const_iterator(const _const_iterator& src) : fIterator(src.fIterator) {}
		_const_iterator(const Array::_iterator& src) : fIterator(src) {}
		
		bool operator == (const _const_iterator& rhs) const { return fIterator == rhs.fIterator; }
		bool operator != (const _const_iterator& rhs) const { return fIterator != rhs.fIterator; }
		bool operator < (const _const_iterator& rhs) const { return fIterator < rhs.fIterator; }
		const Array::reference operator *() const { return *fIterator; }
		const Node* operator ->() const { return fIterator.operator ->(); }
		
		_const_iterator& operator ++() 
		{
			++fIterator; 
			return *this; 
		}
		_const_iterator operator ++(int)
		{ 
			_const_iterator tmp(*this); 
			++fIterator; 
			return tmp;
		}
		_const_iterator& operator --() 
		{
			++fIterator; 
			return *this; 
		}
		_const_iterator operator --(int)
		{ 
			_const_iterator tmp(*this);
			++fIterator; 
			return tmp;
		}
		
		difference_type operator - ( const _const_iterator& rhs ) const 
		{
			return fIterator - rhs.fIterator;
		}
		_const_iterator operator + (difference_type dist) const
		{
			return _const_iterator(fIterator + dist);
		}
		_const_iterator& operator += (difference_type dist) 
		{ 
			fIterator += dist;
			return *this; 
		}
		_const_iterator& operator -= (difference_type dist)
		{ 
			fIterator -= dist;
			return *this; 
		}
	};
#endif // CP_PropertyList_Uses_CoreFoundation
} // end of namespace CP_PropertyList

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_PropertyList_Array
