//=========================================================================
//
//	FILE:				CP_PropertyList_Dictionary.h
//	
//						Created by Jens Miltner on 01.12.04.
//	
//	PURPOSE:			CP_PropertyList dictionary.
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 264 $
//						Latest Checkout:  $Date: 2005-08-28 19:28:53 -0700 (Sun, 28 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_PropertyList_Dictionary
#define _H_CP_PropertyList_Dictionary

#include <iterator>
#include <utility>
#include <map>
#include "CP_PropertyList_Node.h"
#include "CP_PropertyList_Array.h"
#include "CP_PropertyList_String.h"
#include "CP_SmartPtr.h"
#if CP_PropertyList_Uses_CoreFoundation
#include "CP_CFMutableObject.h"
#endif

CPLAT_Begin_Namespace_CPLAT

class CP_Date;
class CP_Time;
class CP_DateTime;
class CP_UUID;

namespace CP_PropertyList {

class CP_EXPORT Dictionary : public Node
#if CP_PropertyList_Uses_CoreFoundation
		, protected CP_CFMutableObject<CFDictionaryRef, CFMutableDictionaryRef>
#endif
{
	public:

	#if CP_PropertyList_Uses_CoreFoundation
		class _value_reference;
		class _iterator;
		class _const_iterator;
		
		typedef CP_String									key_type;
		typedef Node*										mapped_type;
		typedef std::pair<const CP_String, CP_SmartRef<const Node> >	value_type;
		typedef std::pair<const CP_String, _value_reference >			reference;
		typedef std::pair<const CP_String, const _value_reference >		const_reference;
		typedef UInt32										size_type;
		typedef UInt32										difference_type;
		typedef value_type*									pointer;
		typedef const value_type*							const_pointer;
		typedef _iterator									iterator;
		typedef _const_iterator								const_iterator;
		typedef std::reverse_iterator<iterator>				reverse_iterator;
		typedef std::reverse_iterator<const_iterator>		const_reverse_iterator;
	#else
		typedef std::map<const CP_String, CP_SmartRef<const Node> > DictionaryMap;

		typedef DictionaryMap::value_type value_type;
		typedef DictionaryMap::reference reference;
		typedef DictionaryMap::const_reference const_reference;
		typedef DictionaryMap::size_type size_type;
		typedef DictionaryMap::difference_type difference_type;
		typedef DictionaryMap::pointer pointer;
		typedef DictionaryMap::const_pointer const_pointer;
		typedef DictionaryMap::iterator iterator;
		typedef DictionaryMap::const_iterator const_iterator;
		typedef DictionaryMap::reverse_iterator reverse_iterator;
		typedef DictionaryMap::const_reverse_iterator const_reverse_iterator;
		typedef CP_String key_type;
		typedef CP_SmartRef<const Node> mapped_type;
	#endif
		
		Dictionary();
		//explicit Dictionary(const CP_String* inKeys, const Node** inValues);
		Dictionary(const Dictionary& inSrc);
		Dictionary(const CP_String& inKey, const Node& inValue);
	#if CP_PropertyList_Uses_CoreFoundation
		explicit Dictionary(CFDictionaryRef inDictionaryRef, bool inRetain=true);
	#endif
		virtual ~Dictionary() throw();
		
		//! @name Getting / Setting values
		//! @{
		const Node* GetValue(const CP_String& inKey) const;
		void SetValue(const CP_String& inKey, const Node& inValue);
		bool HasKey(const CP_String& inKey) const throw();
		const Array* GetAllKeys() const;
		const Array* GetAllValues() const;
		
		//! @name Convenience Getters/Setters
		//! @{
		//@{
		/*!
			 @brief   Get an optional value.
		  
			 Will retrieve the value associated with the specified key from the dictionary.
			 If dictionary does not contain a value for the key, the function will return \a false,
			 otherwise it will store the value in the \a outValue parameter and return \a true.
		  
			 @exception std::bad_cast	raised if value exists, but is not of the expected type.

			 @param[in]		inKey		The key that specifies which value to retrieve.
			 @param[out]	outValue	If value was found, it will be returned in this output parameter
		  
			 @return					Whether or not the value was found.
		 */
		bool GetStringValue(const CP_String& inKey, CP_String& outValue) const;
		bool GetUInt32Value(const CP_String& inKey, UInt32& outValue) const;
		bool GetSInt32Value(const CP_String& inKey, SInt32& outValue) const;
		bool GetFloatValue(const CP_String& inKey, double& outValue) const;
		bool GetBooleanValue(const CP_String& inKey, bool& outValue) const;
		bool GetDateTimeValue(const CP_String& inKey, CP_DateTime& outDateTime) const;
		bool GetDateTimeValue(const CP_String& inKey, CP_Date* outDate, CP_Time* outTime) const;
		bool GetUUIDValue(const CP_String& inKey, CP_UUID& outUUID) const;
		/*!
			@brief   Get an optional value.
		 
		 Will retrieve the value associated with the specified key from the dictionary.
		 If dictionary does not contain a value for the key, the function will return \a NULL,
		 otherwise it will return a pointer to the value object found.
		 
		 @exception std::bad_cast	raised if value exists, but is not of the expected type.
		 
		 @param		inKey		The key that specifies which value to retrieve.
		 
		 @return				The array value found, or \a NULL if value doesn't exist.
		 */
		const Array* GetArrayValue(const CP_String& inKey) const;
		/*!
			@brief   Get an optional value.
		 
		 Will retrieve the value associated with the specified key from the dictionary.
		 If dictionary does not contain a value for the key, the function will return \a NULL,
		 otherwise it will return a pointer to the value object found.
		 
		 @exception std::bad_cast	raised if value exists, but is not of the expected type.
		 
		 @param		inKey		The key that specifies which value to retrieve.
		 
		 @return				The dictionary value found, or \a NULL if value doesn't exist.
		 */
		const Dictionary* GetDictionaryValue(const CP_String& inKey) const;
		//@}
		//@{
		/*!
			@brief   Get a required value.
		 
			 Will retrieve the value associated with the specified key from the dictionary.
			 If dictionary does not contain a value for the key, an exception will be raised.
			 
			@exception std::bad_cast						raised if value exists, but is not of the expected type.
			@exception CP_OSErrorException(CP_resNotFound)	raised if value does not exist.
			 
			@param		inKey		The key that specifies which value to retrieve.
			 
			@return		The value associated with \a inKey.
		 */
		CP_String GetStringValue(const CP_String& inKey) const;
		UInt32 GetUInt32Value(const CP_String& inKey) const;
		SInt32 GetSInt32Value(const CP_String& inKey) const;
		double GetFloatValue(const CP_String& inKey) const;
		bool GetBooleanValue(const CP_String& inKey) const;
		CP_DateTime GetDateTimeValue(const CP_String& inKey) const;
		CP_Date GetDateValue(const CP_String& inKey) const;
		CP_Time GetTimeValue(const CP_String& inKey) const;
		CP_UUID GetUUIDValue(const CP_String& inKey) const;
		//@}
		//@{
		/*!
			@brief   Set the value for a key.
		 
			 Will set/change the value associated with the specified key in the dictionary.
			 If dictionary does not contain a value for the key, a new value will be inserted,
			 if the dictionary already contains a value for the key, it will be overwritten with
			 the new value.
			 
			 @exception none (except for fatal things like e.g. memory allocation failure, etc.)
			 
			@param		inKey		The key that specifies which value to set.
			@param		inValue		The new value to set.
		 */
		void SetStringValue(const CP_String& inKey, const CP_String& inValue);
		void SetNumberValue(const CP_String& inKey, UInt32 inValue);
		void SetNumberValue(const CP_String& inKey, SInt32 inValue);
		void SetNumberValue(const CP_String& inKey, double inValue);
		void SetBooleanValue(const CP_String& inKey, bool inValue);
		void SetDateTimeValue(const CP_String& inKey, const CP_DateTime& inDateTime);
		void SetDateValue(const CP_String& inKey, const CP_Date& inDate);
		void SetTimeValue(const CP_String& inKey, const CP_Time& inTime);
		void SetUUIDValue(const CP_String& inKey, const CP_UUID& inUUID);
		//@}
		//! @}
		
		//! @}
		
		//! Make a mutable dictionary with the same contents.
		Dictionary* MakeMutable() const;
		
		//! @name STL associative container like interface
		//! @{
		size_type size() const;
		bool empty() const;
		iterator find(const key_type& inKey);
		const_iterator find(const key_type& inKey) const;
		std::pair<iterator, bool> insert(const value_type& inValue);
		iterator begin();
		iterator end();
		const_iterator begin() const;
		const_iterator end() const;
		reverse_iterator rbegin();
		reverse_iterator rend();
		const_reverse_iterator rbegin() const;
		const_reverse_iterator rend() const;
#if CP_PropertyList_Uses_CoreFoundation		
		reference operator [](const key_type& inKey);
#else
		mapped_type& operator [](const key_type& inKey);
#endif		

		//! @}

		
		virtual NodeType_t NodeType() const throw();

		static Dictionary* FromXMLData(CP_InStream& inXMLDataStream);

		virtual void PrintToLog(CP_LogFile& inLogFile, SInt32 inLevel=0, SInt32 inIndentLevel=0) const;
	#if CP_PropertyList_Uses_CoreFoundation
		virtual CFTypeRef/*CFDictionaryRef*/ GetNodeCFRef() const;
	#endif
	#if !CP_PropertyList_Uses_CoreFoundation
	protected:
		//! write XML data for this node only (i.e. without xml header, plist root, etc.)
		virtual void WriteNodeXMLData(CP_OutStream& inXMLDataStream, int inLeadingSpaces) const;
	#endif

		// class data:
	protected:
	#if !CP_PropertyList_Uses_CoreFoundation
		DictionaryMap fDictionaryElements;
	#endif
	};

#if CP_PropertyList_Uses_CoreFoundation
	// Dictionary reference implemenation
	class Dictionary::_value_reference
	{
		friend class Dictionary;
		friend class Dictionary::_iterator;
		friend class Dictionary::_const_iterator;
		
		Dictionary*				fDictionary;
		Dictionary::key_type	fKey;
		CP_SmartRef<const Node> fNode;
	protected:
		_value_reference() {}
		_value_reference(Dictionary* inDictionary, const Dictionary::key_type& inKey) : fDictionary(inDictionary), fKey(inKey) {}
	public:	
		_value_reference(const Node* inNode) : fNode(inNode) {}
		_value_reference(const _value_reference& inSrc) : fDictionary(inSrc.fDictionary), fKey(inSrc.fKey) {}
		
		inline const Node* operator -> () const
		{
			return fNode.Get() ? fNode.Get() : fDictionary->GetValue(fKey); 
		}
		inline operator const Node*() const 
		{ 
			return fNode.Get() ? fNode.Get() : fDictionary->GetValue(fKey); 
		}
		_value_reference& operator = (const Node* inNewValue);
		inline _value_reference& operator = (const _value_reference& inNewValue)
		{ 
			return operator = (static_cast<const Node*>(inNewValue)); 
		}
	};
	
	class Dictionary::_iterator : public std::iterator<std::bidirectional_iterator_tag, Dictionary::value_type, Dictionary::difference_type, Dictionary::value_type*, Dictionary::reference>
	{
		friend class Dictionary;
		friend class Dictionary::_const_iterator;
		
	protected:
		Dictionary*									fDictionary;
		mutable const Array*						fKeys;
		mutable Array::size_type					fCursor;
		mutable Array::size_type					fEndCursor;
		mutable CP_SmartPtr<Dictionary::key_type>	fCursorKey;
		
		struct compare_key
		{
			const Dictionary::key_type&	key;
			compare_key(const Dictionary::key_type& inKey) : key(inKey) {}
			bool operator()(const Node* node) const { return key < *static_cast<const String*>(node); }
		};
		void AssertKeys() const;
		void ResolveCursorKey() const;
		
		_iterator(Dictionary* inDictionary, bool inBegin);
		_iterator(Dictionary* inDictionary, const Dictionary::key_type& inKey);
	public:
		typedef Dictionary::difference_type difference_type;
		
		_iterator() : fDictionary(NULL), fKeys(NULL), fCursor(Array::npos) {}
		_iterator(const _iterator& src);
		~_iterator();
		
		bool operator == (const _iterator& rhs) const;
		bool operator != (const _iterator& rhs) const { return !operator ==(rhs); }

		Dictionary::reference operator *() const;
		
		_iterator& operator ++();
		_iterator operator ++(int);
		_iterator& operator --();
		_iterator operator --(int);
	};
	
	class Dictionary::_const_iterator : public std::iterator<std::bidirectional_iterator_tag, const Dictionary::value_type, Dictionary::difference_type, const Dictionary::value_type*, const Dictionary::reference>
	{
		friend class Dictionary;
	protected:
		Dictionary::_iterator		fIterator;
		
		_const_iterator(const Dictionary* inDictionary, bool inBegin) : fIterator(const_cast<Dictionary*>(inDictionary), inBegin)
		{
		}
		_const_iterator(const Dictionary* inDictionary, const Dictionary::key_type& inKey) : fIterator(const_cast<Dictionary*>(inDictionary), inKey)
		{
		}
	public:
		typedef Dictionary::_iterator::difference_type difference_type;
		
		_const_iterator() {}
		_const_iterator(const _const_iterator& src) : fIterator(src.fIterator)
		{
		}
		_const_iterator(const _iterator& iter) : fIterator(iter)
		{
		}

		bool operator == (const _const_iterator& rhs) const { return fIterator == rhs.fIterator; }
		bool operator != (const _const_iterator& rhs) const { return fIterator != rhs.fIterator; }
		
		const Dictionary::reference operator *() const { return *fIterator; }
		
		_const_iterator& operator ++() 
		{
			++fIterator;
			return *this; 
		}
		_const_iterator operator ++(int)
		{ 
			_const_iterator tmp(*this); 
			++fIterator; 
			return tmp;
		}
		_const_iterator& operator --() 
		{
			--fIterator; 
			return *this; 
		}
		_const_iterator operator --(int)
		{ 
			_const_iterator tmp(*this);
			--fIterator; 
			return tmp;
		}
	};
#endif // CP_PropertyList_Uses_CoreFoundation
	

} // end of namespace CP_PropertyList

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_PropertyList_Dictionary
