//=========================================================================
//
//	FILE:				CP_ResourcePtr.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Template classes that manage the the lifetime of various
//						platform resources, such as menus, memory, images, etc.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_ResourcePtr
#define _H_CP_ResourcePtr
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC
	struct CFObjectTrait
	{
		typedef CFTypeRef				resource_type;

		static void						Retain( resource_type inResourceType );
		static void						Release( resource_type inResourceType );
		static UInt32					Count( resource_type inResourceType );
	};

	struct EventResourceTrait
	{
		typedef EventRef				resource_type;

		static void						Retain( resource_type inResourceType );
		static void						Release( resource_type inResourceType );
		static UInt32					Count( resource_type inResourceType );
	};

	struct MenuResourceTrait
	{
		typedef MenuRef					resource_type;

		static void						Retain( resource_type inResourceType );
		static void						Release( resource_type inResourceType );
		static UInt32					Count( resource_type inResourceType );
	};

	struct IconResourceTrait
	{
		typedef IconRef				resource_type;

		static void						Retain( resource_type inResourceType );
		static void						Release( resource_type inResourceType );
		static UInt32					Count( resource_type inResourceType );
	};

	struct WindowResourceTrait
	{
		typedef WindowRef				resource_type;

		static void						Retain( resource_type inResourceType );
		static void						Release( resource_type inResourceType );
		static UInt32					Count( resource_type inResourceType );
	};

	template <typename T> struct CP_ResourcePtrTrait					{};

		// CFTypes
	template <> struct CP_ResourcePtrTrait<CFBundleRef>					{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFMutableStringRef>			{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFStringRef>					{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFUUIDRef>					{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFURLRef>					{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFDictionaryRef>				{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFMutableDictionaryRef>		{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFArrayRef>					{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFMutableArrayRef>			{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFMutableDataRef>			{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFDataRef>					{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFDateRef>					{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFBooleanRef>				{ typedef CFObjectTrait Type; };
	template <> struct CP_ResourcePtrTrait<CFTimeZoneRef>				{ typedef CFObjectTrait Type; };
		
		// Other types
	template <> struct CP_ResourcePtrTrait<EventRef>					{ typedef EventResourceTrait Type; };
	template <> struct CP_ResourcePtrTrait<IconRef>						{ typedef EventResourceTrait Type; };
	template <> struct CP_ResourcePtrTrait<MenuRef>						{ typedef MenuResourceTrait Type; };
	template <> struct CP_ResourcePtrTrait<WindowRef>					{ typedef WindowResourceTrait Type; };
#endif

#if TARGET_OS_WIN32
	template <typename T> struct CP_ResourcePtrTrait					{};
#endif

#if TARGET_OS_LINUX
	template <typename T> struct CP_ResourcePtrTrait					{};
#endif

template<class T> class CP_ResourcePtr
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ResourcePtr();
										CP_ResourcePtr( const CP_ResourcePtr& inResourcePtr );
	explicit							CP_ResourcePtr(
												T inPtr,
												bool inFromCopyCreate );
	//@}
	//! Destructor.
										~CP_ResourcePtr();
	//@}

	//! @name operators
	//@{
	CP_ResourcePtr&						operator = ( const CP_ResourcePtr& inResourcePtr );

										operator T () const;
	//@}

	//! @name Ptr management
	//@{
			T							Get() const									{ return( fPtr ); }
			void						Reset( T inPtr );
			void						Reset(
												T inPtr,
												bool inFromCopyCreate );
			T							Release();
	//@}

protected:

	// class data
private:
	typedef typename CP_ResourcePtrTrait<T>::Type		Type;

	T									fPtr;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

template<class T>
inline CP_ResourcePtr<T>::CP_ResourcePtr()
	: fPtr(NULL)
{
}

template<class T>
inline CP_ResourcePtr<T>::CP_ResourcePtr( const CP_ResourcePtr& inResourcePtr )
	: fPtr(inResourcePtr.fPtr)
{
	if( fPtr != NULL )
		Type::Retain( fPtr );
}

template<class T>
inline CP_ResourcePtr<T>::CP_ResourcePtr( T inPtr, bool inFromCopyCreate )
	: fPtr(NULL)
{
	Reset( inPtr, inFromCopyCreate );
}

template<class T>
inline CP_ResourcePtr<T>::~CP_ResourcePtr()
{
	if( fPtr != NULL )
	{
		Type::Release( fPtr );
	}
}

template<class T>
void CP_ResourcePtr<T>::Reset( T inPtr )
{
	if( inPtr != NULL )
		Type::Retain( inPtr );
	
	if( fPtr != NULL )
		Type::Release( fPtr );

	fPtr = inPtr;
}

template<class T>
void CP_ResourcePtr<T>::Reset( T inPtr, bool inFromCopyCreate )
{
	if( inPtr != NULL  &&  !inFromCopyCreate )
		Type::Retain( inPtr );
		
	if( fPtr != NULL )
	{
		if( Type::Count( fPtr ) > 0 )
			Type::Release( fPtr );
	}

	fPtr = inPtr;
}

template<class T>
T CP_ResourcePtr<T>::Release()
{
	T		currentPtr = fPtr;
	
	fPtr = NULL;
	
	return( currentPtr );
}

template <typename T>
CP_ResourcePtr<T>& CP_ResourcePtr<T>::operator = ( const CP_ResourcePtr& inPtr )
{
	Reset( inPtr.fPtr );
	
	return( *this );
}

template <typename T>
CP_ResourcePtr<T>::operator T () const
{
	return( fPtr );
}

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ResourcePtr
