//=========================================================================
//
//	FILE:				CP_Socket.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to allow communication via sockets.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 149 $
//						Latest Checkout:  $Date: 2005-04-14 23:11:45 -0700 (Thu, 14 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_Socket
#define _H_CP_Socket
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_String.h"
#include "CP_Types.h"
#include "CP_NetworkAddress.h"

#if TARGET_OS_MAC
	#include <netinet/in.h>
	#include <sys/types.h>
	#include <sys/socket.h>
#endif

#if TARGET_OS_WIN32
		// WinSock version information (use WSA_VERSION for WSAStartup())
	#ifndef MAKEWORD
		#define MAKEWORD(l,h) ((WORD)(((BYTE)(l))|(((WORD)(BYTE)(h))<<8)))
	#endif
	
	#define WSA_MAJOR_VERSION 1
	#define WSA_MINOR_VERSION 1
	#define WSA_VERSION MAKEWORD(WSA_MAJOR_VERSION, WSA_MINOR_VERSION)
#endif

	// typedef to avoid name conflict when used below
typedef	struct sockaddr_in				sockaddr_in;
typedef	struct hostent					hostent;

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Socket
*	@brief		Class to allow communication via sockets.
*/
class CP_EXPORT CP_Socket
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Socket();
										CP_Socket(UInt32 inAddress, UInt16 inPort);
										CP_Socket(const char* inAddress, UInt16 inPort);
										CP_Socket(const CP_NetworkAddress& inAddress, UInt16 inPort);
	//! Destructor.
	virtual								~CP_Socket() throw();
	//@}

	//! @name Connection
	//@{
			void						Connect(UInt32 inTimeoutInSeconds=0);
			void						Connect(UInt32 inAddress, UInt16 inPort,UInt32 inTimeoutInSeconds=0);
			void						Connect(const char* inAddress, UInt16 inPort,UInt32 inTimeoutInSeconds=0);
			void						Connect(const CP_NetworkAddress& inAddress, UInt16 inPort,UInt32 inTimeoutInSeconds=0);
	//@}
						
	//! @name Data Transfer
	//@{
			void						Send( const CP_String& inMessage );
			void						Send(
												const void *inData,
												UInt32 inDataLength );
												
			SInt32						Receive(
												void *inOutBuffer,
												UInt32 inOutBufferLength );

			UInt32 						Available() const;
			bool						HasBytesAvailable() const;
	//@}

	//! @name Getters
	//@{
			static UInt32				GetHostByName(const CP_String& inHostName);
			static CP_NetworkAddress 	GetHostAddressByName(const CP_String& inHostName);

										/*!
										*	Return an UInt32 containing the address we are communicating over.
										*
										*	@exception	none
										*	
										*	@return		UInt32		The return address.
										*/
			UInt32						GetAddress() const							{ return fConnectedSocketAddress.GetIPv4Address(); }

										/*!
										*	Return an GetNetworkAddress containing the address we are communicating over.
										*
										*	@exception	none
										*	
										*	@return		GetNetworkAddress		The return address.
										*/
			CP_NetworkAddress			GetNetworkAddress() const					{ return fConnectedSocketAddress;}

										/*!
										*	Return an UInt32 containing the peer address we are communicating with.
										*
										*	@exception	none
										*	
										*	@return		UInt32		The return peer address.
										*/
			UInt32						GetPeerAddress() const						{ return fConnectedPeerAddress.GetIPv4Address(); }

										/*!
										*	Return a CP_NetworkAddress containing the peer address we are communicating with.
										*
										*	@exception	none
										*	
										*	@return		CP_NetworkAddress		The return peer address.
										*/
			CP_NetworkAddress			GetPeerNetworkAddress() const 				{ return fConnectedPeerAddress;}

										/*!
										*	Returns the port we are communicating over.
										*
										*	@exception	none
										*	
										*	@return		UInt16		The port we are communicating over.
										*/
			UInt16						GetPort() const 							{ return fConnectedSocketPort; }

										/*!
										*	Returns the port of the peer are communicating with.
										*
										*	@exception	none
										*	
										*	@return		UInt16		The peer port we are communicating with.
										*/
			UInt16						GetPeerPort() const 						{ return fConnectedPeerPort; }

										/*!
										*	Returns the name of the host as a string.
										*
										*	@exception	none
										*	
										*	@return		CP_String	The name of the host.
										*/
			CP_String					GetHostName()								{ return( fHostName ); }
			
										/*!
										*	Return an UInt32 containing the peer address we are communicating with.
										*
										*	@exception	none
										*	
										*	@return		UInt32		The return peer address.
										*/
	//@}

	//! @name Misc
	//@{
			UInt32						GetHostID();
	//@}

	//! @name Terminate
	//@{
			void						Close();
			void 						CloseInput();
			void 						CloseOutput();
	//@}

	//! @name Testers
	//@{
										/*!
										*	Return true if we have a current connection.
										*
										*	@exception	none
										*	
										*	@return		bool			True if we have a connection.
										*/
			bool 						IsConnected( char rdwr = -1 ) const;
			bool						IsConnectionClosed(UInt32 inTimeoutInMicroseconds=10000) const;
	//@}

protected:
	
	void								Initialize(void);
	void								GetConnectedSocketInfo(void);

private:

	// class data
protected:
										
	CP_String							fHostName;				/*! CP_String containing the name of the host. */
	char								fIsConnected;			/*! flag to indicate if we are currently connected. */
										
	sockaddr_in							fSocketAddress;			/*! a sockaddr_in structure with the current address info. */
	hostent								*fnlp_host;
	
	#if TARGET_OS_MAC || TARGET_OS_LINUX
	int									fSocketDescriptor;
	#endif

	#if TARGET_OS_WIN32
	SOCKET								fSocketDescriptor;
	#endif
	
	CP_NetworkAddress					fConnectedSocketAddress;
	UInt16								fConnectedSocketPort;
	CP_NetworkAddress					fConnectedPeerAddress;
	UInt16								fConnectedPeerPort;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Socket( int socketDescriptor );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Socket( SOCKET socketDescriptor );
	//@}
#endif

#if TARGET_OS_LINUX
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Socket( int socketDescriptor );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Socket


