//=========================================================================
//
//	FILE:				CP_UndoAction.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Action class that provide undo / redo support.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_UndoAction
#define _H_CP_UndoAction
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Action.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_UndoAction
*	@brief		CP_Action class that provide undo / redo support.
*/
class CP_EXPORT CP_UndoAction : public CP_Action
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_UndoAction( const CP_String& inMenuText );
	//! Destructor.
										~CP_UndoAction() throw();
	//@}
						
	//! @name Undo / Redo
	//@{
	virtual	bool						DoUndo();
	virtual	bool						DoRedo();
	//@}
	
	//! @name Edit Menu
	//@{
										/*!
										*	Return the menu text to be displayed for this command.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The text to be used.
										*/
			CP_String					GetMenuText()								{ return( fMenuText ); }

										/*!
										*	Sets the menu text to be displayed for this command.
										*	
										*	@param		inString		The text to be displayed in the menu.	
										*
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetMenuText(const CP_String& inString )		{ fMenuText = inString; }
	//@}

	//! @name Perform
	//@{
	virtual	void						Finish( CP_UndoAction *inCommand );
	//@}
	
	//! @name Testers
	//@{
										/*!
										*	Return true if this command has been done.
										*	
										*	@exception	none
										*	
										*	@return		bool		True if the action has been done.
										*/
	virtual	bool						IsDone()									{ return( fDone ); }

										/*!
										*	Return true if this action is undoable.
										*	
										*	@exception	none
										*	
										*	@return		bool		True if the action is undoable.
										*/
	virtual	bool						IsUndoable()								{ return( fIsUndoable ); }

										/*!
										*	Return true if this action is redoable.
										*	
										*	@exception	none
										*	
										*	@return		bool		True if the action is redoable.
										*/
	virtual	bool						IsRedoable()								{ return( fIsUndoable ); }
	//@}

protected:
	//! @name Undo / Redo
	//@{
										/*!
										*	Undo the command. Subclasses must override.
										*	
										*	@exception	none
										*	
										*	@return		bool		True if the action was performed.
										*							Default implementation returns false.
										*/
	virtual	bool						UndoSelf()									{ return( false ); }

										/*!
										*	Redo the command. Subclasses must override.
										*	
										*	@exception	none
										*	
										*	@return		bool		True if the action was performed.
										*							Default implementation returns false.
										*/
	virtual	bool						RedoSelf()									{ return( false ); }
	//@}
	
	//! @name Perform
	//@{
										/*!
										*	Finish the action. This is called just before the action is deleted.
										*
										*	@param		inCommand		The Action or Command to act upon.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual void						FinishSelf( CP_UndoAction */*inCommand*/ )	{}
	//@}
	
	//! @name Setters
	//@{
			void						SetIsUndoable( bool isUndoable = true );
			void						SetIsRedoable( bool isRedoable = true );
	//@}
			
	// class data
protected:
										
	CP_String							fMenuText;		/*! Text displayed in the undo menu item. */
										
	bool								fDone;			/*! Flag to indicate if the command is finished. */
	bool								fIsUndoable;	/*! Flag to indicate if the command is undoable. */
	bool								fIsRedoable;	/*! Flag to indicate if the command is redoable. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_UndoAction

